@@ -2,13 +2,44 @@ import logging as log
 from os import path
 import shutil
 
-from .env_config import OS, PWD, XDG_CONFIG_DIR, HOME_DIR, load_cfg
+from .constants import OS, PWD, XDG_CONFIG_DIR, HOME_DIR
 from .console_logging import print_header
 from .console_logging import print_sub_header as sub_header
+from .env_config import load_cfg
 from .subproc import subproc
 
 
-def run_preinstall_cmds(cmd_list=[], pkg_groups=[]):
+def load_packages_config() -> dict:
+    """
+    Checks if user has local config file before procceding with default config
+    """
+    # check to make sure the user didn't pass in their own packages.yaml
+    usr_pkg_config = path.join(XDG_CONFIG_DIR, 'packages.yaml')
+    if path.exists(usr_pkg_config):
+        return load_cfg(usr_pkg_config)
+    else:
+        default_config = path.join(PWD, 'config/packages.yaml')
+        return load_cfg(default_config)
+
+
+def rotate_github_ssh_keys() -> None:
+    """
+    update SSH pub keys for github.com
+    """
+    log.info("Rotating github.com ssh keys, just in case...")
+    # deletes all keys starting with github.com from ~/.ssh/known_hosts
+    subproc(["ssh-keygen -R github.com"])
+
+    # gets the new public keys from github.com
+    github_keys = subproc(["ssh-keyscan github.com"])
+
+    # the new github.com keys are not automatically added :( so we do it here
+    with open(path.join(HOME_DIR, '.ssh/known_hosts'), 'a') as known_hosts:
+        for line in github_keys.split('/n'):
+            known_hosts.write(line)
+
+
+def run_preinstall_cmds(cmd_list: list, pkg_groups: list) -> None:
     """
     takes a list of package manager pre-install commands and runs them
     if second list of package groups contains gaming, runs additional commands
@@ -34,10 +65,8 @@ def run_preinstall_cmds(cmd_list=[], pkg_groups=[]):
             subproc([cmd_list[pre_cmd]], spinner=SPINNER)
             sub_header(f"[b]{pre_cmd.title()}[/b] completed.")
 
-    return True
 
-
-def run_pkg_mngrs(pkg_mngrs=[], pkg_groups=[]):
+def run_pkg_mngrs(pkg_mngrs: list, pkg_groups=[]) -> None:
     """
     Installs brew and pip3.11 packages. Also apt, snap, and flatpak on Linux.
     Takes optional variables:
@@ -45,35 +74,17 @@ def run_pkg_mngrs(pkg_mngrs=[], pkg_groups=[]):
       - pkg_mngrs: list of package managers to run
     Returns True
     """
-    # check to make sure the user didn't pass in their own packages.yaml
-    user_packages = path.join(XDG_CONFIG_DIR, 'packages.yaml')
-    if path.exists(user_packages):
-        pkg_mngrs_list_of_dicts = load_cfg(user_packages)
-    else:
-        default_config = path.join(PWD, 'config/packages.yaml')
-        pkg_mngrs_list_of_dicts = load_cfg(default_config)
+    log.debug(f"passed in pkg_mngrs: {pkg_mngrs}\npkg_groups: {pkg_groups}")
 
-    log.debug(f"passed in pkg_mngrs: {pkg_mngrs}")
-    log.debug(f"passed in pkg_groups: {pkg_groups}")
-
-    log.info("Rotating github.com ssh keys, just in case...")
-
-    # deletes all keys starting with github.com from ~/.ssh/known_hosts
-    subproc(["ssh-keygen -R github.com"])
-
-    # gets the new public keys from github.com
-    github_keys = subproc(["ssh-keyscan github.com"])
-
-    # the new github.com keys are not automatically added :( so we do it here
-    with open(path.join(HOME_DIR, '.ssh/known_hosts'), 'a') as known_hosts:
-        for line in github_keys.split('/n'):
-            known_hosts.write(line)
+    rotate_github_ssh_keys()
 
+    pkg_mngrs_list_of_dicts = load_packages_config()
     # we iterate through pkg_mngrs which should already be sorted
     for pkg_mngr in pkg_mngrs:
-
         pkg_mngr_dict = pkg_mngrs_list_of_dicts[pkg_mngr]
+
         available_pkg_groups = pkg_mngr_dict['packages']
+        log.debug(f"pkg groups for {pkg_mngr} are {available_pkg_groups}")
 
         # brew has a special flow because it works on both linux and mac
         if pkg_mngr == 'brew':
@@ -83,8 +94,6 @@ def run_pkg_mngrs(pkg_mngrs=[], pkg_groups=[]):
                         pkg_groups = list(pkg_groups)
                     pkg_groups.append("macOS")
 
-        log.debug(f"pkg groups for {pkg_mngr} are {available_pkg_groups}")
-
         # make sure that the package manager has any groups that were passed in
         if any(check in pkg_groups for check in available_pkg_groups):
 
@@ -101,12 +110,17 @@ def run_pkg_mngrs(pkg_mngrs=[], pkg_groups=[]):
                          "https://snapcraft.io/docs/installing-snap-on-debian")
                 # continues onto the next package manager
                 continue
-            else:
-                # run package manager specific setup if needed: update/upgrade
-                run_preinstall_cmds(pkg_cmds, pkg_groups)
+
+            if pkg_mngr == 'brew':
+                # this installs macOS brew taps
+                install_brew_taps(pkg_mngr_dict['taps']['macOS'])
+
+            # run package manager specific setup if needed: update/upgrade
+            run_preinstall_cmds(pkg_cmds, pkg_groups)
 
             # run the list command for the given package manager
             list_cmd = pkg_cmds['list']
+            # TODO: figure out a way to make this less hacky
             if pkg_mngr == 'apt':
                 list_cmd = path.join(PWD, list_cmd)
             list_pkgs = subproc([list_cmd], quiet=True)
@@ -122,7 +136,9 @@ def run_pkg_mngrs(pkg_mngrs=[], pkg_groups=[]):
                 # if package group is in the packages.yaml file
                 if pkg_group in available_pkg_groups:
                     if pkg_group == "macOS":
+                        # zathura needs some help on macOS
                         check_zathura()
+
                     install_pkg_group(pkg_cmds['install'],
                                       available_pkg_groups[pkg_group],
                                       installed_pkgs)
@@ -132,10 +148,10 @@ def run_pkg_mngrs(pkg_mngrs=[], pkg_groups=[]):
             if 'cleanup' in pkg_cmds:
                 subproc([pkg_cmds['cleanup']])
                 sub_header("[b]Cleanup[/b] step Completed.")
-    return True
 
 
-def install_pkg_group(install_cmd="", pkgs_to_install=[], installed_pkgs=[]):
+def install_pkg_group(install_cmd: str, pkgs_to_install: list,
+                      installed_pkgs: list) -> None:
     """
     Installs packages if they are not already installed.
     provided install command string.
@@ -171,10 +187,26 @@ def install_pkg_group(install_cmd="", pkgs_to_install=[], installed_pkgs=[]):
 
         # Actual installation
         subproc([install_cmd + pkg], quiet=True)
-    return True
 
 
-def check_zathura():
+def install_brew_taps(taps: list) -> None:
+    """
+    Checks current brew taps, and then runs brew tap {tap} on any taps that are
+    in a list of git repos from packages.yaml, and aren't already tapped
+    """
+    current_taps = subproc(["brew tap"]).split('\n')
+    log.debug(f"taps list is: {taps}")
+    log.debug(f"Current taps are: {current_taps}")
+
+    # for each tap, complete cmd by prepending `brew tap`
+    for index, tap in enumerate(taps):
+        log.debug(f"index: {index} tap: {tap}")
+        # only brew tap if they don't already exist
+        if tap not in current_taps:
+            subproc(["brew tap " + tap])
+
+
+def check_zathura() -> None:
     """
     make sure zathura is installed on macos
     installs via brew if it's not installed
@@ -182,11 +214,7 @@ def check_zathura():
     """
     if not shutil.which("zathura"):
         zathura_pdf = "$(brew --prefix zathura-pdf-mupdf)"
-        cmds = ["brew tap zegervdv/zathura",
-                "brew install zathura",
-                "brew install zathura-pdf-mupdf",
-                "mkdir -p $(brew --prefix zathura)/lib/zathura",
+        cmds = ["mkdir -p $(brew --prefix zathura)/lib/zathura",
                 f"ln -s {zathura_pdf}/libpdf-mupdf.dylib" +
                 f"{zathura_pdf}/lib/zathura/libpdf-mupdf.dylib"]
         subproc(cmds, quiet=True)
-    return True
